@@ -1,8 +1,7 @@
-
 /*
- * libspeechd.c - Shared library for easy acces to Speech Dispatcher functions
+  libspeechd.c - Shared library for easy acces to Speech Dispatcher functions
  *
- * Copyright (C) 2001, 2002, 2003, 2006 Brailcom, o.p.s.
+ * Copyright (C) 2001, 2002, 2003, 2006, 2007 Brailcom, o.p.s.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: libspeechd.c,v 1.29 2006/11/29 16:56:11 hanke Exp $
+ * $Id: libspeechd.c,v 1.32 2007-06-16 21:11:04 hanke Exp $
  */
 
 
@@ -51,6 +50,11 @@
 
 /* --------------  Private functions headers ------------------------*/
 
+typedef struct {
+    char *str;
+    size_t bytes;
+}_SPDString;
+
 static int spd_set_priority(SPDConnection* connection, SPDPriority priority);
 static char* escape_dot(const char *otext);
 static int isanum(char* str);		
@@ -63,6 +67,10 @@ static void xfree(void *ptr);
 static int ret_ok(char *reply);
 static void SPD_DBG(char *format, ...);
 static void* spd_events_handler(void*);
+static _SPDString spd_string_append(_SPDString str, char *tail, int bytes);
+static _SPDString spd_string_new(void);
+
+pthread_mutex_t spd_logging_mutex;
 
 /* --------------------- Public functions ------------------------- */
 
@@ -114,13 +122,19 @@ spd_open(const char* client_name, const char* connection_name, const char* user_
 #ifdef LIBSPEECHD_DEBUG
     spd_debug = fopen("/tmp/libspeechd.log", "w");
     if (spd_debug == NULL) SPD_FATAL("COULDN'T ACCES FILE INTENDED FOR DEBUG");
+
+    ret = pthread_mutex_init(&spd_logging_mutex, NULL);
+    if(ret != 0){
+      fprintf(stderr, "Mutex initialization failed");
+      exit(1);
+    }
     SPD_DBG("Debugging started");
 #endif /* LIBSPEECHD_DEBUG */
     
   /* Connect to server */
     ret = connect(connection->socket, (struct sockaddr *)&address, sizeof(address));
     if (ret == -1){
-	SPD_DBG("Error: Can't connect to server: %s", strerror(errno));
+        SPD_DBG("Error: Can't connect to server: %s", strerror(errno));
 	return NULL;
     }
 
@@ -254,7 +268,7 @@ spd_say(SPDConnection *connection, SPDPriority priority, const char* text)
 
     msg_id = get_param_int(reply, 1, &err);
     if (err < 0){
-	SPD_DBG("Can't determine SSIP message unique ID parameter.");
+       SPD_DBG("Can't determine SSIP message unique ID parameter.");
     }
     xfree(reply);
 
@@ -679,12 +693,12 @@ spd_set_notification(SPDConnection *connection, SPDNotification notification, co
     if (connection->mode != SPD_MODE_THREADED) return -1;
 
     if (state == NULL){
-	SPD_DBG("Requested state is NULL");
-	return -1;
+      SPD_DBG("Requested state is NULL");
+      return -1;
     }
     if (strcmp(state, "on") && strcmp(state, "off")){
-	SPD_DBG("Invalid argument for spd_set_notification: %s", state);
-	return -1;
+      SPD_DBG("Invalid argument for spd_set_notification: %s", state);
+      return -1;
     }
 
     pthread_mutex_lock(connection->ssip_mutex);
@@ -771,7 +785,6 @@ spd_execute_command_with_reply(SPDConnection *connection, char* command, char **
 {
     char *buf;    
     int r;
-    
     buf = g_strdup_printf("%s\r\n", command);
     *reply = spd_send_data_wo_mutex(connection, buf, SPD_WAIT_REPLY);
     xfree(buf);
@@ -816,11 +829,16 @@ spd_send_data_wo_mutex(SPDConnection *connection, const char *message, int wfr)
 	    pthread_cond_wait(connection->cond_reply_ready, connection->mutex_reply_ready);
 	    pthread_mutex_unlock(connection->mutex_reply_ready);
 	    /* Read the reply */
-	    reply = connection->reply;
-	    connection->reply = NULL;
+	    if (connection->reply != NULL){
+		reply = strdup(connection->reply);
+	    }else{
+	        SPD_DBG("Error: Can't read reply, broken socket.");
+		return NULL;
+	    }
+	    xfree(connection->reply);
 	    bytes = strlen(reply);
 	    if (bytes == 0){
-		SPD_DBG("Error: Can't read reply, broken socket.");
+	        SPD_DBG("Error: Empty reply, broken socket.");
 		return NULL;
 	    }
 	    /* Signal the reply has been read */
@@ -834,8 +852,8 @@ spd_send_data_wo_mutex(SPDConnection *connection, const char *message, int wfr)
     }else{
 	if (connection->mode == SPD_MODE_THREADED)
 	    pthread_mutex_unlock(connection->mutex_reply_ready);
-        SPD_DBG("<< : no reply expected");
-        return "NO REPLY";
+	SPD_DBG("<< : no reply expected");
+        return strdup("NO REPLY");
     } 
 
     return reply;
@@ -857,8 +875,8 @@ spd_set_priority(SPDConnection *connection, SPDPriority priority)
     case SPD_NOTIFICATION: strcpy(p_name, "NOTIFICATION"); break;
     case SPD_PROGRESS: strcpy(p_name, "PROGRESS"); break;
     default: 
-        SPD_DBG("Error: Can't set priority! Incorrect value.");
-        return -1;
+      SPD_DBG("Error: Can't set priority! Incorrect value.");
+      return -1;
     }
 		 
     sprintf(command, "SET SELF PRIORITY %s", p_name);
@@ -882,7 +900,8 @@ get_reply(SPDConnection *connection)
     do{
 	bytes = getline(&line, &N, connection->stream);	
 	if (bytes == -1){
-	    SPD_FATAL("Error: Can't read reply, broken socket!");		
+	    SPD_DBG("Error: Can't read reply, broken socket!");
+	    return NULL;
 	}
 	g_string_append(str, line);
 	/* terminate if we reached the last line (without '-' after numcode) */
@@ -906,9 +925,13 @@ spd_events_handler(void* conn)
 
 	/* Read the reply/event (block if none is available) */
 	reply = get_reply(connection);
-        SPD_DBG("<< : |%s|\n", reply);
-
-	reply_code = get_err_code(reply);
+	if (reply == NULL){
+	    SPD_DBG("ERROR: BROKEN SOCKET");
+	    reply_code = -1;
+	}else{
+	    SPD_DBG("<< : |%s|\n", reply);
+	    reply_code = get_err_code(reply);
+	}
 
 	if ((reply_code >= 700) && (reply_code < 800)){
 	    int msg_id;
@@ -921,12 +944,12 @@ spd_events_handler(void* conn)
 	    /* Extract message id */
 	    msg_id = get_param_int(reply, 1, &err);
 	    if (err < 0){
-		SPD_DBG("Bad reply from Speech Dispatcher: %s (code %d)", reply, err);
+	      SPD_DBG("Bad reply from Speech Dispatcher: %s (code %d)", reply, err);
 		break;
 	    }
 	    client_id = get_param_int(reply, 2, &err);
 	    if (err < 0){
-		SPD_DBG("Bad reply from Speech Dispatcher: %s (code %d)", reply, err);
+	      SPD_DBG("Bad reply from Speech Dispatcher: %s (code %d)", reply, err);
 		break;
 	    }
 	    /*  Decide if we want to call a callback */
@@ -945,7 +968,7 @@ spd_events_handler(void* conn)
 		int err;
 		im = get_param_str(reply, 3, &err);
 		if ((err < 0) || (im == NULL)){
-		    SPD_DBG("Broken reply from Speech Dispatcher: %s", reply);
+		  SPD_DBG("Broken reply from Speech Dispatcher: %s", reply);
 		    break;
 		}
 		/* Call the callback */
@@ -957,7 +980,11 @@ spd_events_handler(void* conn)
 	    /* This is a protocol reply */
 	    pthread_mutex_lock(connection->mutex_reply_ready);
 	    /* Prepare the reply to the reply buffer in connection */
-	    connection->reply = reply;
+	    if (reply != NULL){
+		connection->reply = strdup(reply);
+	    }else{
+		connection->reply = NULL;
+	    }
 	    /* Signal the reply is available on the condition variable */
 	    /* this order is correct and necessary */
 	    pthread_cond_signal(connection->cond_reply_ready);
@@ -966,6 +993,7 @@ spd_events_handler(void* conn)
 	    /* Wait until it has bean read */
 	    pthread_cond_wait(connection->cond_reply_ack, connection->mutex_reply_ack);
 	    pthread_mutex_unlock(connection->mutex_reply_ack);
+	    xfree(reply);
 	    /* Continue */	
 	}
     }
@@ -1077,8 +1105,8 @@ get_err_code(char *reply)
     if(isanum(err_code)){
         err = atoi(err_code);
     }else{
-        SPD_DBG("ret_ok: not a number\n");
-        return -1;	
+      SPD_DBG("ret_ok: not a number\n");
+      return -1;	
     }
 
     return err;
@@ -1118,74 +1146,91 @@ xfree(void *ptr)
         free(ptr);
 }
 
+static _SPDString
+spd_string_new(void)
+{
+    _SPDString result;
+    
+    result.bytes=1024;
+    result.str=malloc(result.bytes);
+    result.str[0]='\0';
+    
+    return result;
+}
+
+static _SPDString
+spd_string_append(_SPDString str, char *tail, int bytes)
+{
+    if (tail == NULL) return str;
+    
+    while ((strlen(str.str)+strlen(tail))>= str.bytes){
+	str.bytes *= 2;
+	str.str = realloc(str.str, str.bytes);
+    }
+
+    if (bytes != -1)
+	strncat(str.str, tail, bytes);
+    else
+	strcat(str.str, tail);
+
+    return str;
+}
+   
+
 static char*
 escape_dot(const char *text)
 {
-    char *seq;
-    GString *ntext;
-    char *p;
-    char *otext;
-    char *ret = NULL;
-    int len;
+    _SPDString result;
+    size_t text_len;
+    int pos;
+    char *place;
 
-    if (otext == NULL) return NULL;
+    result=spd_string_new();
 
-    SPD_DBG("Incomming text to escaping: |%s|", otext);
+    if (text == NULL) return NULL;
+    text_len = strlen(text);
 
-    p = (char*) text;
-    otext = p;
-
-    ntext = g_string_new("");
-
-    len = strlen(text);
-    if (len == 1){
-        if (!strcmp(text, ".")){
-            g_string_append(ntext, "..");
-            p += 1;
-        }
+    pos = 0;
+    if (text_len == 0){
+	return result.str;
     }
-    else if (len >= 3){
-        if ((p[0] == '.') && (p[1] == '\r') && (p[2] == '\n')){
-            g_string_append(ntext, "..\r\n");
-            p += 3;
-        }
+    /*If text contains only a dot, escape it*/
+    else if (text_len==1 && text[0]=='.'){
+	result = spd_string_append(result, "..", -1);
+	return result.str;
+    }
+    /*Escape the leading '.\r\n', if present*/
+    else if (text_len >=3){
+	if (!strncmp(text, ".\r\n", 3)){
+	    result = spd_string_append(result, "..", -1);
+	    pos += 1;
+	}
     }
 
-    //    SPD_DBG("Altering text (I): |%s|", ntext->str);
-
-    while (seq = strstr(p, "\r\n.\r\n")){
-        assert(seq>p);
-        g_string_append_len(ntext, p, seq-p);
-        g_string_append(ntext, "\r\n..\r\n");
-        p = seq+5;
+    /* Replace all full \r\n.\r\n sequences */
+    while((place = strstr(&text[pos], "\r\n.\r\n"))){
+	assert (place >= text);
+	int n = place-&text[pos];
+	result = spd_string_append(result, (char*) &text[pos], n);
+	result = spd_string_append(result, "\r\n..", -1);
+	/* Only rewind before the following \r\n to resume search for
+	   sequence */
+	pos +=n+3;
     }
 
-    //   SPD_DBG("Altering text (II): |%s|", ntext->str);    
-
-    len = strlen(p);
-    if (len >= 3){
-        if ((p[len-3] == '\r') && (p[len-2] == '\n')
-            && (p[len-1] == '.')){
-            g_string_append(ntext, p);
-            g_string_append(ntext, ".");
-            p += len;
-            //  SPD_DBG("Altering text (II-b): |%s|", ntext->str);    
-        }
+    /* Paste the rest */    
+    result=spd_string_append(result, (char*) &text[pos], -1);
+    
+    /*Escape the trailing '\r\n.' if found by adding an additional dot at the
+      end*/
+    if(strlen(result.str)>=3){
+	if(!strncmp(&result.str[strlen(result.str)-3], "\r\n.", 3)){
+	    result=spd_string_append(result, ".", -1);
+	    return result.str;
+	}
     }
 
-    if (p == otext){
-        SPD_DBG("No escaping needed.");
-        g_string_free(ntext, 1);
-        return NULL;
-    }else{
-        g_string_append(ntext, p);
-        ret = ntext->str;
-        g_string_free(ntext, 0);
-    }
-
-    SPD_DBG("Altered text after escaping: |%s|", ret);
-
-    return ret;
+    return result.str;
 }
 
 #ifdef LIBSPEECHD_DEBUG
@@ -1194,11 +1239,13 @@ SPD_DBG(char *format, ...)
 {
         va_list args;
 
-        va_start(args, format);
-        vfprintf(spd_debug, format, args);
-        va_end(args);
+	pthread_mutex_lock(&spd_logging_mutex);    	
+	va_start(args, format);
+	vfprintf(spd_debug, format, args);
+	va_end(args);
 	fprintf(spd_debug, "\n");
 	fflush(spd_debug);
+	pthread_mutex_unlock(&spd_logging_mutex);    	
 }
 #else  /* LIBSPEECHD_DEBUG */
 static void
