@@ -1,4 +1,3 @@
-
 /*
  * festival.c - Speech Dispatcher backend for Festival
  *
@@ -19,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: festival.c,v 1.71 2007/02/27 15:53:35 hanke Exp $
+ * $Id: festival.c,v 1.80 2007-11-26 14:41:03 hanke Exp $
  */
 
 #include "fdset.h"
@@ -57,6 +56,8 @@ AudioID *festival_audio_id = NULL;
 AudioOutputType festival_audio_output_method;
 char *festival_pars[10];
 
+VoiceDescription** festival_voice_list = NULL;
+
 enum{
     FCT_SOCKET = 0,
     FCT_LOCAL = 1,
@@ -79,6 +80,7 @@ void festival_parent_clean();
 void festival_set_rate(signed int rate);
 void festival_set_pitch(signed int pitch);
 void festival_set_voice(EVoiceType voice);
+void festival_set_synthesis_voice(char* synthesis_voice);
 void festival_set_language(char* language);
 void festival_set_punctuation_mode(EPunctMode punct);
 void festival_set_cap_let_recogn(ECapLetRecogn recogn);
@@ -104,6 +106,11 @@ MOD_OPTION_1_STR(FestivalAudioOutputMethod);
 MOD_OPTION_1_STR(FestivalOSSDevice);
 MOD_OPTION_1_STR(FestivalALSADevice);
 MOD_OPTION_1_STR(FestivalNASServer);
+MOD_OPTION_1_STR(FestivalPulseServer);
+MOD_OPTION_1_INT(FestivalPulseMaxLength);
+MOD_OPTION_1_INT(FestivalPulseTargetLength);
+MOD_OPTION_1_INT(FestivalPulsePreBuffering);
+MOD_OPTION_1_INT(FestivalPulseMinRequest);
 
 MOD_OPTION_1_INT(FestivalCacheOn);
 MOD_OPTION_1_INT(FestivalCacheMaxKBytes);
@@ -164,6 +171,11 @@ module_load(void)
     MOD_OPTION_1_STR_REG(FestivalOSSDevice, "/dev/dsp");
     MOD_OPTION_1_STR_REG(FestivalALSADevice, "default");
     MOD_OPTION_1_STR_REG(FestivalNASServer, NULL);
+    MOD_OPTION_1_STR_REG(FestivalPulseServer, "default");
+    MOD_OPTION_1_INT_REG(FestivalPulseMaxLength, 132300);
+    MOD_OPTION_1_INT_REG(FestivalPulseTargetLength, 4410);
+    MOD_OPTION_1_INT_REG(FestivalPulsePreBuffering, 2200);
+    MOD_OPTION_1_INT_REG(FestivalPulseMinRequest, 880);
 
     MOD_OPTION_1_INT_REG(FestivalCacheOn, 1);
     MOD_OPTION_1_INT_REG(FestivalCacheMaxKBytes, 5120);
@@ -224,6 +236,9 @@ module_init(char **status_info)
 	}
     }
 
+    /* Get festival voice list */
+    festival_voice_list = festivalGetVoices(festival_info);
+
     DBG("Openning audio output system");
     if (!strcmp(FestivalAudioOutputMethod, "oss")){
 	DBG("Using OSS audio output method");
@@ -246,9 +261,20 @@ module_init(char **status_info)
 	festival_audio_id = spd_audio_open(AUDIO_NAS, (void**) festival_pars, &error);
 	festival_audio_output_method = AUDIO_NAS;
     }
+    else if (!strcmp(FestivalAudioOutputMethod, "pulse")){
+	DBG("Using PulseAudio output method");
+	festival_pars[0] = (void *) FestivalPulseServer;
+	festival_pars[1] = (void *) FestivalPulseMaxLength;
+	festival_pars[2] = (void *) FestivalPulseTargetLength;
+	festival_pars[3] = (void *) FestivalPulsePreBuffering;
+	festival_pars[4] = (void *) FestivalPulseMinRequest;
+	festival_pars[5] = NULL;
+	festival_audio_id = spd_audio_open(AUDIO_PULSE, (void**) festival_pars, &error);
+	festival_audio_output_method = AUDIO_PULSE;
+    }
     else{
 	ABORT("Sound output method specified in configuration not supported. "
-	      "Please choose 'oss', 'alsa' or 'nas'.");
+	      "Please choose 'oss', 'alsa', 'nas' or 'pulse'.");
     }
     if (festival_audio_id == NULL){
 	g_string_append_printf(info, "Opening sound device failed. Reason: %s. ", error);
@@ -282,10 +308,17 @@ module_init(char **status_info)
 
     *status_info = info->str;
     g_string_free(info, 0);
+
     return 0;
 }
 #undef ABORT
 
+VoiceDescription**
+module_list_voices(void)
+{
+  return festival_voice_list;
+}
+
 int
 module_speak(char *data, size_t bytes, EMessageType msgtype)
 {
@@ -328,16 +361,20 @@ module_speak(char *data, size_t bytes, EMessageType msgtype)
     }
 
     /* If the voice was changed, re-set all the parameters */
-    if ((msg_settings.voice != msg_settings_old.voice) 
-        || strcmp(msg_settings.language, msg_settings_old.language)){
+    // TODO: Handle synthesis_voice change too 
+    if ((msg_settings.voice != msg_settings_old.voice)
+	|| (msg_settings.language != NULL) && (msg_settings_old.language!=NULL) &&
+	(strcmp(msg_settings.language, msg_settings_old.language))){
 	DBG("Cleaning old settings table");
         CLEAN_OLD_SETTINGS_TABLE();
     }
 
+
     /* Setting voice parameters */
     DBG("Updating parameters");
     UPDATE_STRING_PARAMETER(language, festival_set_language);
     UPDATE_PARAMETER(voice, festival_set_voice);
+    UPDATE_STRING_PARAMETER(synthesis_voice, festival_set_synthesis_voice);
     UPDATE_PARAMETER(rate, festival_set_rate);
     UPDATE_PARAMETER(pitch, festival_set_pitch);
     UPDATE_PARAMETER(volume, festival_set_volume);
@@ -487,7 +524,14 @@ festival_send_to_audio(FT_Wave *fwave)
     
     if (track.samples != NULL){
 	DBG("Sending to audio");
-	ret = spd_audio_play(festival_audio_id, track);
+		switch (audio_endian){
+			case SPD_AUDIO_LE:
+				ret = spd_audio_play(festival_audio_id, track, SPD_AUDIO_LE);
+				break;
+			case SPD_AUDIO_BE:
+				ret = spd_audio_play(festival_audio_id, track, SPD_AUDIO_BE);
+				break;
+		}
 	if (ret < 0) DBG("ERROR: Can't play track for unknown reason.");
 	DBG("Sent to audio.");
     }
@@ -651,7 +695,7 @@ _festival_speak(void* nothing)
 	    }       	
 
 	    if (fwave->num_samples != 0){
-		DBG("Sending message to audio: %d bytes\n",
+		DBG("Sending message to audio: %ld bytes\n",
 		    (fwave->num_samples) * sizeof(short));	
     
 		if(FestivalDebugSaveOutput){
@@ -702,6 +746,8 @@ void
 festival_set_language(char* language)
 {   
     FestivalSetLanguage(festival_info, language, NULL);
+    xfree(festival_voice_list);
+    festival_voice_list = festivalGetVoices(festival_info);
 }
 
 void
@@ -712,9 +758,13 @@ festival_set_voice(EVoiceType voice)
     voice_name = EVoice2str(voice);
     FestivalSetVoice(festival_info, voice_name, NULL);
     xfree(voice_name);
+}
 
-    /* Because the new voice can use diferent bitrate */
-    DBG("Filling new sample wave\n");
+void
+festival_set_synthesis_voice(char *voice_name)
+{
+
+  FestivalSetSynthesisVoice(festival_info, voice_name);
 }
 
 void
