@@ -18,11 +18,11 @@
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  * 02110-1301, USA.
  *
- * $Id: module_main.c,v 1.10 2006/07/11 16:12:27 hanke Exp $
+ * $Id: module_main.c,v 1.13 2007-09-24 06:51:58 hanke Exp $
  */
 
 /* So that gcc doesn't comply */
-int getline(char**, int*, FILE*);
+int getline(char**, size_t*, FILE*);
 
 #define PROCESS_CMD(command, function) \
 if (!strcmp(cmd_buf, #command"\n")){ \
@@ -51,7 +51,7 @@ main(int argc, char *argv[])
     char *cmd_buf;
     int ret;
     int ret_init;
-    int n;
+    size_t n;
     char *configfilename;
     char *status_info;        
 
@@ -135,6 +135,7 @@ main(int argc, char *argv[])
         else PROCESS_CMD(KEY, do_key)
         else PROCESS_CMD_NRP(STOP, do_stop) 
         else PROCESS_CMD_NRP(PAUSE, do_pause) 
+        else PROCESS_CMD(LIST VOICES, do_list_voices)
         else PROCESS_CMD(SET, do_set) 
         else PROCESS_CMD(QUIT, do_quit) 
         else{
