@@ -1,7 +1,7 @@
 /*
  * module_utils.c - Module utilities
  *           Functions to help writing output modules for Speech Dispatcher
- * Copyright (C) 2003,2006 Brailcom, o.p.s.
+ * Copyright (C) 2003,2006, 2007 Brailcom, o.p.s.
  *
  * This is free software; you can redistribute it and/or modify it under the
  * terms of the GNU Lesser General Public License as published by the Free
@@ -18,7 +18,7 @@
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
  * 02110-1301, USA.
  *
- * $Id: module_utils.c,v 1.43 2006/07/11 16:12:27 hanke Exp $
+ * $Id: module_utils.c,v 1.50 2007-11-26 17:07:18 hanke Exp $
  */
 
 #include "fdsetconv.h"
@@ -65,7 +65,7 @@ do_message(EMessageType msgtype)
     int ret;
     char *cur_line;
     GString *msg;
-    int n;
+    size_t n;
     int nlines = 0;
 
     msg = g_string_new("");
@@ -97,7 +97,12 @@ do_message(EMessageType msgtype)
     if ((msgtype != MSGTYPE_TEXT) && (nlines > 2)){
         return strdup("305 DATA MORE THAN ONE LINE");
     }
-  
+
+    if ((msgtype == MSGTYPE_CHAR) && (!strcmp(msg->str,"space"))){
+	g_string_free(msg, 1);
+	msg = g_string_new(" ");
+    }
+
     ret = module_speak(msg->str, strlen(msg->str), msgtype);
 
     g_string_free(msg,1);
@@ -180,7 +185,7 @@ do_set(void)
     char *cur_value = NULL;
     char *line = NULL;
     int ret;
-    int n;
+    size_t n;
     int number; char *tptr;
     int err = 0;                /* Error status */
 
@@ -208,6 +213,7 @@ do_set(void)
             else SET_PARAM_STR_C(spelling_mode, str2ESpellMode)
             else SET_PARAM_STR_C(cap_let_recogn, str2ECapLetRecogn)
             else SET_PARAM_STR_C(voice, str2EVoice)
+            else SET_PARAM_STR(synthesis_voice)
             else SET_PARAM_STR(language)
             else err=2;             /* Unknown parameter */
         }
@@ -220,6 +226,37 @@ do_set(void)
     
     return strdup("401 ERROR INTERNAL"); /* Can't be reached */
 }
+
+char *
+do_list_voices(void)
+{
+  VoiceDescription **voices;
+  int i;
+  char *lang, *dialect;
+  GString *voice_list;
+
+  voices = module_list_voices();
+  if (voices == NULL){
+    return strdup("304 CANT LIST VOICES");
+  }
+  
+  voice_list = g_string_new("");
+  for (i=0; ;i++){
+    if (voices[i] == NULL) break;
+    if (voices[i]->language==NULL) lang=strdup("none");
+    else lang=strdup(voices[i]->language);
+    if (voices[i]->dialect==NULL) dialect=strdup("none");
+    else dialect=strdup(voices[i]->dialect);
+    g_string_append_printf(voice_list, "200-%s %s %s\n", voices[i]->name, lang, dialect);
+    xfree(lang); xfree(dialect);
+  }
+  g_string_append(voice_list, "200 OK VOICE LIST SENT");
+
+  DBG("Voice prepared to  sens to speechd");
+
+  return voice_list->str;
+}
+
 #undef SET_PARAM_NUM
 #undef SET_PARAM_STR
 
@@ -342,7 +379,6 @@ module_strip_ssml(char *message)
 	    omit = 0;
 	    continue;
 	}
-	    
 	if (!strncmp(&(message[i]), "&lt;", 4)){ 
 	    i+=3;
 	    out[n++] = '<';
@@ -351,9 +387,13 @@ module_strip_ssml(char *message)
 	    i+=3;
 	    out[n++] = '>';
 	}
-	else if (!omit) out[n++] = message[i];
+	else if (!strncmp(&(message[i]), "&amp;", 5)){
+	    i+=4;
+	    out[n++] = '&';
+	}
+	else if (!omit || i == len) out[n++] = message[i];
     }
-    DBG("in stripping ssml: |%s|", out);
+    DBG("In stripping ssml: |%s|", out);
 
     return out;
 }
@@ -557,7 +597,7 @@ module_parent_dp_write(TModuleDoublePipe dpipe, const char *msg, size_t bytes)
 {
     int ret;
     assert(msg != NULL);
-    DBG("going to write %d bytes", bytes);
+    DBG("going to write %ld bytes", bytes);
     ret = write(dpipe.pc[1], msg, bytes);      
     DBG("written %d bytes", ret);
     return ret;
