@@ -2,7 +2,7 @@
 /*
  * module.c - Output modules for Speech Dispatcher
  *
- * Copyright (C) 2001, 2002, 2003, 2006 Brailcom, o.p.s.
+ * Copyright (C) 2001, 2002, 2003, 2006, 2007 Brailcom, o.p.s.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: module.c,v 1.31 2006/07/20 15:32:48 hanke Exp $
+ * $Id: module.c,v 1.36 2007-11-26 14:24:21 hanke Exp $
  */
 
 #define _GNU_SOURCE
@@ -47,6 +47,7 @@ load_output_module(char* mod_name, char* mod_prog, char* mod_cfgfile, char* mod_
     char *arg1;
     int cfg = 0;
     int ret;
+    char *module_conf_dir;
 
     if (mod_name == NULL) return NULL;
     
@@ -54,7 +55,13 @@ load_output_module(char* mod_name, char* mod_prog, char* mod_cfgfile, char* mod_
 
     module->name = (char*) spd_strdup(mod_name);
     module->filename = (char*) spd_get_path(mod_prog, MODULEBINDIR);    
-    module->configfilename = (char*) spd_get_path(mod_cfgfile, MODULECONFDIR);
+    
+    module_conf_dir = g_strdup_printf("%s/modules/",
+				      SpeechdOptions.conf_dir);
+
+    module->configfilename = (char*) spd_get_path(mod_cfgfile, module_conf_dir);
+    g_free(module_conf_dir);
+
     if (mod_dbgfile != NULL) module->debugfilename = strdup(mod_dbgfile);
     else module->debugfilename = NULL;
 
@@ -185,6 +192,8 @@ load_output_module(char* mod_name, char* mod_prog, char* mod_cfgfile, char* mod_
 		}
 
 	    }
+	    /* Get a list of supported voices */
+	    _output_get_voices(module);
 	    fclose(f);
 	    g_string_append_printf(reply, "---------------\n");
 
@@ -248,7 +257,6 @@ reload_output_module(OutputModule *old_module)
     }
 
     g_hash_table_replace(output_modules, new_module->name, new_module);
-    
     destroy_module(old_module);
 
     return 0;
