@@ -18,7 +18,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: options.c,v 1.9 2006/07/25 11:46:19 hanke Exp $
+ * $Id: options.c,v 1.11 2007-11-26 14:24:32 hanke Exp $
  */
 
 /* NOTE: Be careful not to include options.h, we would
@@ -41,6 +41,7 @@ options_print_help(char *argv[])
     "-l, --log-level      -      Set log level (1..5)\n"
     "-p, --port           -      Specify a port number\n"
     "-P, --pid-file       -      Set path to pid file\n"
+    "-C, --config-dir     -      Set path to configuration\n"
     "-v, --version        -      Report version of this program\n"
     "-h, --help           -      Print this info\n\n"
     "Copyright (C) 2003,2006 Brailcom, o.p.s.\n"
@@ -107,6 +108,9 @@ options_parse(int argc, char *argv[])
 	case 'P':
 	    SPD_OPTION_SET_STR(pid_file);
 	    break;
+	case 'C':
+	    SPD_OPTION_SET_STR(conf_dir);
+	    break;
         case 'v':
             options_print_version();
             exit(0);
