@@ -18,10 +18,12 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: speechd.c,v 1.68 2007/02/17 18:55:56 hanke Exp $
+ * $Id: speechd.c,v 1.75 2007-11-26 14:24:39 hanke Exp $
  */
 
 #include <gmodule.h>
+#include <sys/stat.h>
+
 
 #include "speechd.h"
 
@@ -37,6 +39,9 @@
 #include "options.h"
 #include "server.h"
 
+/* Glib g_makedir */
+
+int  g_mkdir      (const gchar *filename, int mode);
 /* Manipulating pid files */
 int create_pid_file();
 void destroy_pid_file();
@@ -81,18 +86,17 @@ MSG2(int level, char *kind, char *format, ...)
     int custom_log = (kind != NULL && custom_log_kind != NULL &&
                       !strcmp(kind, custom_log_kind) &&
                       custom_logfile != NULL);
-    
+
     if(std_log || custom_log) {
         va_list args;
         va_list args2;
         int i;
-	
-        if(std_log) {
-            va_start(args, format);
-        }
-        if(custom_log) {
-            va_start(args2, format);
-        }
+
+	pthread_mutex_lock(&logging_mutex);    	
+
+        if(std_log) va_start(args, format);        
+        if(custom_log) va_start(args2, format);
+        
         {
             {
                 /* Print timestamp */
@@ -100,18 +104,21 @@ MSG2(int level, char *kind, char *format, ...)
                 char *tstr;
 		struct timeval tv;
                 t = time(NULL);
-                tstr = strdup(ctime(&t));
-		gettimeofday(&tv,NULL);
-                /* Remove the trailing \n */
-                assert(strlen(tstr)>1);
-                tstr[strlen(tstr)-1] = 0;
+		tstr = strdup(ctime(&t));
+		gettimeofday(&tv,NULL);                
+		assert(tstr);
+		/* Remove the trailing \n */
+		assert(strlen(tstr)>1);
+		tstr[strlen(tstr)-1] = 0;
                 if(std_log) {
-                    fprintf(logfile, "[%s : %d] speechd: ",
-			    tstr, (int) tv.tv_usec);
+		  fprintf(logfile, "[%s : %d] speechd: ",
+			  tstr, (int) tv.tv_usec); 
+		    //		  fprintf(logfile, "[test : %d] speechd: ",
+		    //			   (int) tv.tv_usec); 
                 }
                 if(custom_log) {
-                    fprintf(custom_logfile, "[%s : %d] speechd: ",
-			    tstr, (int) tv.tv_usec);
+		  fprintf(custom_logfile, "[%s : %d] speechd: ",
+			  tstr, (int) tv.tv_usec);
                 }
 		spd_free(tstr);
             }
@@ -140,7 +147,8 @@ MSG2(int level, char *kind, char *format, ...)
         if(custom_log) {
             va_end(args2);
         }
-    }				
+	pthread_mutex_unlock(&logging_mutex);		
+    }		
 }
 
 /* The main logging function for Speech Dispatcher,
@@ -150,11 +158,12 @@ MSG2(int level, char *kind, char *format, ...)
 void
 MSG(int level, char *format, ...)
 {
+
     if(level <= SpeechdOptions.log_level) {
         va_list args;
         int i;
-		
-        va_start(args, format);
+	pthread_mutex_lock(&logging_mutex);		
+	va_start(args, format);
         {
             {
                 /* Print timestamp */
@@ -162,24 +171,27 @@ MSG(int level, char *format, ...)
                 char *tstr;
 		struct timeval tv;
                 t = time(NULL);
-                tstr = strdup(ctime(&t));
+		tstr = strdup(ctime(&t));
 		gettimeofday(&tv,NULL);
                 /* Remove the trailing \n */
-                assert(strlen(tstr)>1);
-                tstr[strlen(tstr)-1] = 0;
-                fprintf(logfile, "[%s : %d] speechd: ",
+		assert(tstr);
+		assert(strlen(tstr)>1);
+		tstr[strlen(tstr)-1] = 0;
+		fprintf(logfile, "[%s : %d] speechd: ",
 			tstr, (int) tv.tv_usec);
+		/*                fprintf(logfile, "[%s : %d] speechd: ",
+				  tstr, (int) tv.tv_usec);*/
 		spd_free(tstr);
             }
             for(i=1;i<level;i++){
-                fprintf(logfile, " ");
+	      fprintf(logfile, " ");
             }
             vfprintf(logfile, format, args);
             fprintf(logfile, "\n");
             fflush(logfile);
         }
         va_end(args);
-
+	pthread_mutex_unlock(&logging_mutex);		
     }				
 }
 
@@ -356,27 +368,26 @@ speechd_options_init(void)
 {
     SpeechdOptions.log_level_set = 0;
     SpeechdOptions.port_set = 0;
+    SpeechdOptions.localhost_access_only_set = 0;
     SpeechdOptions.pid_file = NULL;
+    SpeechdOptions.conf_file = NULL;
+    SpeechdOptions.home_speechd_dir = NULL;
+    SpeechdOptions.log_dir = NULL;
     spd_mode = SPD_MODE_DAEMON;    
 }
 
 
 void
-speechd_init(void)
+speechd_init()
 {
     int START_NUM_FD = 16;
     int ret;
     int i;
-
+    
 
     SpeechdStatus.max_uid = 0;
     SpeechdStatus.max_gid = 0;
 
-    /* Initialize logging */
-    logfile = stdout;
-    custom_logfile = NULL;
-    custom_log_kind = NULL;
-
     /* Initialize inter-thread comm pipe */
     if (pipe(speaking_pipe))
 	{
@@ -391,6 +402,7 @@ speechd_init(void)
     /* Initialize lists */
     MessagePausedList = NULL;
     message_history = NULL;
+    output_modules_list = NULL;
 
     /* Initialize hash tables */
     fd_settings = g_hash_table_new(g_int_hash,g_int_equal);
@@ -439,9 +451,23 @@ speechd_init(void)
     ret = pthread_mutex_init(&socket_com_mutex, NULL);
     if(ret != 0) DIE("Mutex initialization failed");
 
+    if (SpeechdOptions.log_dir == NULL){
+      if (SpeechdOptions.home_speechd_dir){
+	SpeechdOptions.log_dir = g_strdup_printf("%s/log/",
+						 SpeechdOptions.home_speechd_dir);
+	mkdir(g_path_get_dirname(SpeechdOptions.log_dir), S_IRWXU);
+      }else{
+	SpeechdOptions.log_dir = strdup("/var/log/speech-dispatcher/");
+      }
+    }
+
     /* Load configuration from the config file*/
+    MSG(4, "Reading Speech Dispatcher configuration from %s",
+	SpeechdOptions.conf_file);
     speechd_load_configuration(0);
 
+    logging_init();
+
     /* Check for output modules */
     if (g_hash_table_size(output_modules) == 0){
         DIE("No speech output modules were loaded - aborting...");
@@ -459,7 +485,6 @@ speechd_init(void)
 void
 speechd_load_configuration(int sig)
 {
-    char *configfilename = SYS_CONF"/speechd.conf";
     configfile_t *configfile = NULL;
 
     /* Clean previous configuration */
@@ -480,13 +505,18 @@ speechd_load_configuration(int sig)
     spd_options = add_config_option(spd_options, &spd_num_options, "", 0,
                                     NULL, NULL, 0);
     
-    configfile = dotconf_create(configfilename, spd_options, 0, CASE_INSENSITIVE);
-    if (!configfile) DIE ("Error opening config file\n");
+    configfile = dotconf_create(SpeechdOptions.conf_file, spd_options, 0, CASE_INSENSITIVE);
+    if (!configfile){
+      MSG(1, "Can't open %s", SpeechdOptions.conf_file);
+      DIE ("Error opening config file\n");
+    }
+    configfile->includepath = strdup(SpeechdOptions.conf_dir);
+    MSG(5, "Config file include path is: %s", configfile->includepath);
     if (dotconf_command_loop(configfile) == 0) DIE("Error reading config file\n");
     dotconf_cleanup(configfile);
 
     free_config_options(spd_options, &spd_num_options);
-    MSG(2,"Configuration has been read from \"%s\"", configfilename);
+    MSG(2,"Configuration has been read from \"%s\"", SpeechdOptions.conf_file);
 }
 	
 void
@@ -591,11 +621,36 @@ create_pid_file()
 }
 
 void
-destroy_pid_file()
+destroy_pid_file(void)
 {
     unlink(SpeechdOptions.pid_file);
 }
 
+void
+logging_init(void)
+{
+  char *file_name = g_strdup_printf("%s/speechd.log", SpeechdOptions.log_dir);    
+  assert(file_name != NULL);
+  if (!strncmp(file_name,"stdout",6)){
+    logfile = stdout;
+    return;
+  }
+  if (!strncmp(file_name,"stderr",6)){
+    logfile = stderr;
+    return;
+  }
+  logfile = fopen(file_name, "a");
+  if (logfile == NULL){
+    fprintf(stderr, "Error: can't open logging file %s! Using stdout.\n",
+	    file_name);
+    logfile = stdout;
+  }
+
+  MSG(2,"Speech Dispatcher Logging to file %s", file_name);
+  return;
+
+}
+
 
 /* --- MAIN --- */
 
@@ -609,15 +664,72 @@ main(int argc, char *argv[])
 
     speechd_options_init();
 
+    /* Initialize logging */
+    logfile = stderr;
+    custom_logfile = NULL;
+    custom_log_kind = NULL;
+
     options_parse(argc, argv);
 
-    if (SpeechdOptions.pid_file == NULL){
-	if (!strcmp(PIDPATH, ""))
-	    SpeechdOptions.pid_file = strdup("/var/run/speech-dispatcher.pid");
-	else
-	    SpeechdOptions.pid_file = strdup(PIDPATH"speech-dispatcher.pid");
+    /* Check if there is .speech-dispatcher directory
+       in user's home directory. If yes, put everything
+       here */
+    {
+      char *home_dir;
+      GDir *testing;
+      home_dir = (char*) g_get_home_dir();
+      if (home_dir){
+	SpeechdOptions.home_speechd_dir = g_strdup_printf("%s/.speech-dispatcher/",
+							  home_dir);
+	MSG(4, "Home dir found, trying to find %s", SpeechdOptions.home_speechd_dir);
+	testing =  g_dir_open(SpeechdOptions.home_speechd_dir, 0, NULL);
+	if (testing){
+	  /* Ok, directory exists */
+	  g_dir_close(testing);
+	  MSG(4, "Using home directory: %s for configuration, pidfile and logging",
+	      SpeechdOptions.home_speechd_dir);
+	}else{
+	  MSG(1, "Directory .speech-dispatcher not found in home");
+	  g_free(SpeechdOptions.home_speechd_dir);
+	  SpeechdOptions.home_speechd_dir = NULL;
+	}
+	g_free(home_dir);
+      }
     }
 
+    /* Initialize logging mutex to workaround ctime threading bug */
+    /* Must be done no later than here */
+    ret = pthread_mutex_init(&logging_mutex, NULL);
+    if(ret != 0){
+      fprintf(stderr, "Mutex initialization failed");
+      exit(1);
+    }
+
+    if (SpeechdOptions.pid_file == NULL){
+      if (SpeechdOptions.home_speechd_dir){	
+	SpeechdOptions.pid_file = g_strdup_printf("%s/pid/speech-dispatcher.pid",
+						  SpeechdOptions.home_speechd_dir);
+	mkdir(g_path_get_dirname(SpeechdOptions.pid_file), S_IRWXU);
+      }else if (!strcmp(PIDPATH, ""))
+	SpeechdOptions.pid_file = strdup("/var/run/speech-dispatcher.pid");
+      else
+	SpeechdOptions.pid_file = strdup(PIDPATH"speech-dispatcher.pid");
+    }
+    
+    if (SpeechdOptions.conf_dir == NULL){
+      if (SpeechdOptions.home_speechd_dir){
+	SpeechdOptions.conf_dir = g_strdup_printf("%sconf/",
+						  SpeechdOptions.home_speechd_dir);
+      }else if (!strcmp(SYS_CONF, "")){
+	SpeechdOptions.conf_dir = strdup("/etc/speech-dispatcher/");
+      }else{
+	SpeechdOptions.conf_dir = strdup(SYS_CONF);
+      }
+    }
+    SpeechdOptions.conf_file = g_strdup_printf("%s/speechd.conf",
+					       SpeechdOptions.conf_dir);
+    /* This is needed for DotConf so that it knows where to include files from */
+
     if (create_pid_file() != 0) exit(1);
 
     /* Register signals */
@@ -638,7 +750,14 @@ main(int argc, char *argv[])
     }
 
     server_address.sin_family = AF_INET;
-    server_address.sin_addr.s_addr = htonl(INADDR_ANY);
+
+    /* Enable access only to localhost or for any address
+       based on LocalhostAccessOnly configuration option. */
+    if (SpeechdOptions.localhost_access_only)
+      server_address.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+    else
+      server_address.sin_addr.s_addr = htonl(INADDR_ANY);
+
     server_address.sin_port = htons(SpeechdOptions.port);
 
     MSG(3,"Openning socket connection");
