@@ -2,7 +2,7 @@
 /*
  * speechd.h - Speech Dispatcher header
  *
- * Copyright (C) 2001, 2002, 2003, 2006 Brailcom, o.p.s.
+ * Copyright (C) 2001, 2002, 2003, 2006, 2007 Brailcom, o.p.s.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: speechd.h,v 1.57 2006/09/28 13:33:15 hanke Exp $
+ * $Id: speechd.h,v 1.62 2007-11-23 18:07:13 hanke Exp $
  */
 
 #ifndef SPEECHDH
@@ -107,8 +107,13 @@ typedef struct{
 
 struct{
     int port, port_set;
+    int localhost_access_only, localhost_access_only_set;
     int log_level, log_level_set;
     char *pid_file;
+    char *conf_file;
+    char *conf_dir;
+    char *home_speechd_dir;
+    char *log_dir;
     int max_history_messages;	/* Maximum of messages in history before they expire */
 }SpeechdOptions;
 
@@ -121,6 +126,7 @@ struct{
 
 /* speak() thread defined in speaking.c */
 pthread_t speak_thread;
+pthread_mutex_t logging_mutex;
 pthread_mutex_t element_free_mutex;
 pthread_mutex_t output_layer_mutex;
 pthread_mutex_t socket_com_mutex;
@@ -132,6 +138,7 @@ int speaking_sem_id;
 
 /* Table of all configured (and succesfully loaded) output modules */
 GHashTable *output_modules;	
+GList *output_modules_list;
 /* Table of settings for each active client (=each active socket)*/
 GHashTable *fd_settings;	
 /* Table of default output modules for different languages */
@@ -207,8 +214,9 @@ void speechd_options_init(void);
 void speechd_init(void);
 void speechd_load_configuration(int sig);
 void speechd_quit(int sig);
-int create_pid_file();
-void destroy_pid_file();
+int create_pid_file(void);
+void destroy_pid_file(void);
 
+void logging_init(void);
 
 #endif
